/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package baza;

import domen.Korisnik;
import domen.Mesto;
import domen.OpstiDomenskiObjekat;
import domen.Proizvod;
import domen.Racun;
import domen.StavkaRacuna;
import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author student1
 */
public class DatabaseBroker {
    private Connection connection;
    private static DatabaseBroker instance;
    
    private DatabaseBroker() {
    }
    
    public static DatabaseBroker getInstance() {
        if (instance == null) {
            instance = new DatabaseBroker();
        }
        return instance;
    }
    
    public void ucitajDriver() throws RuntimeException {
        try {
            Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
//            driver je klasa
        } catch (Exception e) {
            throw new RuntimeException("Neuspesno ucitavanje drivera!");
        }
    }
    
    public void otvoriKonekciju() throws RuntimeException {
        try {
            connection = DriverManager.getConnection("jdbc:odbc:d2", "", "");
//            jdbc:protokol:naziv_baze
            connection.setAutoCommit(false); //zahteva eksplicitnu potvrdu transakcije
        } catch (Exception e) {
            throw new RuntimeException("Neuspesno otvaranje konekcije!");
        }
    }
    
    public void commitTransakcije() throws RuntimeException {
        try {
            connection.commit();
        } catch (SQLException ex) {
            throw new RuntimeException("Neuspesan commit transakcije!");
        }
    }
    
    public void rollbackTransakcije() throws RuntimeException {
        try {
            connection.rollback();
        } catch (SQLException ex) {
            throw new RuntimeException("Neuspesan rollback transakcije!");
        }
    }
    
    public void zatvoriKonekciju() throws RuntimeException {
        try {
            connection.close();
        } catch (Exception e) {
            throw new RuntimeException("Neuspesno zatvaranje konekcije!");
        }
    }
    
//    public void sacuvajMesto(Mesto m) throws RuntimeException{
//        try {
//            String upit = "INSERT INTO Mesto(ptt, naziv) VALUES (" + m.getPtt() + ",'" + m.getNaziv() + "')";
//            System.out.println(upit);
//            Statement st = connection.createStatement();
//            st.executeUpdate(upit);
////            za insert, update, delete -> executeUpdate
////            za select -> executeQuery
//            
//        } catch (SQLException ex) {
//            throw new RuntimeException("Neuspesno cuvanje mesta!");
//        }
//        
//    }
    public void sacuvajMesto(Mesto m) throws RuntimeException{
        try {
            String upit = "INSERT INTO Mesto values (?,?)";
            PreparedStatement ps = connection.prepareStatement(upit);
            //ps.setLong(1, m.getPtt());
            ps.setBigDecimal(1, new BigDecimal(m.getPtt()));
            ps.setString(2, m.getNaziv());
            ps.executeUpdate();
//            ukoliko se koristi PreparedStatement ide se bez parametra na executeUpdate
            
        } catch (SQLException ex) {
            ex.printStackTrace();
            throw new RuntimeException("Nemoguce cuvanje mesta u bazi!");
        }
    }
    
    public List<Mesto> vratiMesta() throws RuntimeException {
        List<Mesto> lm = new ArrayList<>();
        try {
            String upit = "SELECT * FROM Mesto";
            Statement st = connection.createStatement();
            ResultSet rs = st.executeQuery(upit);
            while(rs.next()) {
                String naziv = rs.getString("naziv");
                long ptt = rs.getLong(1);
                Mesto m = new Mesto(ptt, naziv);
                lm.add(m);
            }
        } catch (SQLException ex) {
            throw new RuntimeException("Nemoguce vratiti mesta!");
        }
        
        return lm;
    }
    
    public void sacuvajKorisnika(Korisnik k) throws RuntimeException {
        try {
            String sql = "INSERT INTO Korisnik VALUES (?,?,?,?,?,?,?,?)";
            PreparedStatement ps = connection.prepareStatement(sql);
            ps.setString(1, k.getJmbg());
            ps.setString(2, k.getIme());
            ps.setString(3, k.getPrezime());
            ps.setString(4, k.getKorisnickoIme());
            ps.setString(5, k.getKorisnickaLozinka());
            ps.setString(6, k.getAdresa().getNazivUlice());
            ps.setString(7, k.getAdresa().getBroj());
            ps.setBigDecimal(8, new BigDecimal(k.getAdresa().getMesto().getPtt()));
            ps.executeUpdate();
            ps.close();
        } catch (SQLException ex) {
            ex.printStackTrace();
            throw new RuntimeException("Neuspesno cuvanje korisnika!");
        }
    }

    public List<Korisnik> vratiKorisnike() throws RuntimeException {
        try {
            String sql = "SELECT * FROM Korisnik k INNER JOIN Mesto m ON (k.Ptt=m.Ptt)";
            Statement sqlNaredba = connection.createStatement();
            ResultSet rs = sqlNaredba.executeQuery(sql);
            List<Korisnik> lk = new ArrayList<>();
            while (rs.next()) {
                Korisnik k = new Korisnik();
                k.setJmbg(rs.getString("JMBG"));
                k.setIme(rs.getString("Ime"));
                k.setPrezime(rs.getString("Prezime"));
                k.setKorisnickoIme(rs.getString("KorisnickoIme"));
                k.setKorisnickaLozinka(rs.getString("KorisnickaLozinka"));
                k.getAdresa().setBroj(rs.getString("Broj"));
                k.getAdresa().setNazivUlice(rs.getString("NazivUlice"));
                
                Mesto m = new Mesto();
                m.setNaziv(rs.getString("Naziv"));
                m.setPtt(rs.getLong(8));
                k.getAdresa().setMesto(m);
                
                lk.add(k);
            }
            return lk;
        } catch (SQLException ex) {
            ex.printStackTrace();
            throw new RuntimeException("Neuspesno ucitavanje korisnika!");
        }
    }

    public List<Proizvod> vratiProizvode() throws RuntimeException {
        try {
            String sql = "SELECT * FROM Proizvod";
            Statement sqlNaredba = connection.createStatement();
            ResultSet rs = sqlNaredba.executeQuery(sql);
            List<Proizvod> lp = new ArrayList<>();
            while (rs.next()) {
                Proizvod p = new Proizvod();
                p.setProizvodID(rs.getInt("ProizvodID"));
                p.setNaziv(rs.getString("Naziv"));
                p.setCena(rs.getDouble("Cena"));
                p.setKolicinaNaStanju(rs.getInt("KolicinaNaStanju"));
                lp.add(p);
            }
            return lp;
        } catch (SQLException ex) {
            ex.printStackTrace();
            throw new RuntimeException("Neuspesno ucitavanje proizvoda!");
        }
    }

    public void sacuvajRacun(Racun r) throws RuntimeException {
        try {
            String sql = "INSERT INTO Racun VALUES (?,?,?,?)";
            PreparedStatement sqlNaredba = connection.prepareStatement(sql);
            sqlNaredba.setInt(1, r.getRacunID());
            sqlNaredba.setDate(2, new java.sql.Date(r.getDatum().getTime()));
            sqlNaredba.setDouble(3, r.getUkupanIznos());
            sqlNaredba.setString(4, r.getKorisnik().getJmbg());
            sqlNaredba.executeUpdate();
            
            sql = "INSERT INTO StavkaRacuna VALUES (?,?,?,?,?)";
            sqlNaredba = connection.prepareStatement(sql);
            for (StavkaRacuna s : r.getListaStavki()) {
                sqlNaredba.setInt(1, r.getRacunID());
                sqlNaredba.setInt(2, s.getRbStavke());
                sqlNaredba.setInt(3, s.getKolicina());
                sqlNaredba.setDouble(4, s.getIznos());
                sqlNaredba.setInt(5, s.getProizvod().getProizvodID());
                sqlNaredba.executeUpdate();
            }
            sqlNaredba.close();
        } catch (SQLException ex) {
            ex.printStackTrace();
            throw new RuntimeException("Neuspesno cuvanje racuna!");
        }
    }

    public void vratiSifruRacuna(Racun r) throws RuntimeException {
        try {
            String sql = "SELECT MAX(RacunID) AS Sifra FROM Racun";
            Statement sqlNaredba = connection.createStatement();
            ResultSet rs = sqlNaredba.executeQuery(sql);
            int racunID = 0;
            if (rs.next()) {
                racunID = rs.getInt("Sifra");
            }
            racunID++;
            r.setRacunID(racunID);
            rs.close();
            sqlNaredba.close();
        } catch (SQLException ex) {
            ex.printStackTrace();
            throw new RuntimeException("Neuspesno kreiranje sifre racuna!");
        }
    }

    public void azurirajStanjeProizvoda(Racun r) throws RuntimeException {
        try {
            String sql = "UPDATE Proizvod SET KolicinaNaStanju = ? WHERE ProizvodID = ?";
            PreparedStatement sqlNaredba = connection.prepareStatement(sql);
            for (StavkaRacuna s : r.getListaStavki()) {
                Proizvod p = s.getProizvod();
                sqlNaredba.setInt(1, p.getKolicinaNaStanju());
                sqlNaredba.setInt(2, p.getProizvodID());
                sqlNaredba.executeUpdate();
            }
            sqlNaredba.close();
        } catch (SQLException ex) {
            ex.printStackTrace();
            throw new RuntimeException("Neuspesno azuriranje proizvoda!");
        }
    }
    
    public void sacuvaj(OpstiDomenskiObjekat odo) throws RuntimeException {
        try {
            String sql = "INSERT INTO " + odo.vratiNazivTabele() + " VALUES (" + odo.vratiInsertVrednosti() + ")";
            System.out.println(sql);
            Statement sqlNaredba = connection.createStatement();
            sqlNaredba.executeUpdate(sql);
            sqlNaredba.close();
        } catch (SQLException ex) {
            ex.printStackTrace();
            throw new RuntimeException("Neuspesno cuvanje objekta!");
        }
    }
    
    public List<OpstiDomenskiObjekat> vratiListu(OpstiDomenskiObjekat odo) throws RuntimeException {
        try {
            String sql = "SELECT * FROM " + odo.vratiNazivTabele();
            System.out.println(sql);
            Statement sqlNaredba = connection.createStatement();
            ResultSet rs = sqlNaredba.executeQuery(sql);
            return odo.vratiListu(rs);
        } catch (Exception ex) {
            ex.printStackTrace();
            throw new RuntimeException("Neuspesno ucitavanje objekata!");
        }
    }
} 
