package server;

import baza.RadSaBazom;
import domen.Korisnik;
import domen.Mesto;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.sql.SQLException;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import transfer.TOKlijentZahtev;
import transfer.TOServerOdgovor;
import util.Util;

/**
 *
 * @author student1
 */
public class Server {

    public static void main(String[] args) throws IOException, ClassNotFoundException {
        Server server = new Server();
        server.pokreniServer();
    }

    private void pokreniServer() throws IOException, ClassNotFoundException {
        ServerSocket serverSocket = new ServerSocket(9000);
        Socket soket = serverSocket.accept();
        pokreniKomunikaciju(soket);
    }

    private void pokreniKomunikaciju(Socket soket) throws IOException, ClassNotFoundException {
        while (true) {
            ObjectInputStream inputStream
                    = new ObjectInputStream(soket.getInputStream());
            TOKlijentZahtev klijentZahtev
                    = (TOKlijentZahtev) inputStream.readObject();
            obradiZahtev(klijentZahtev, soket);
        }
    }

    private void obradiZahtev(TOKlijentZahtev klijentZahtev, Socket soket) throws IOException {
        int operacija = klijentZahtev.getOperacija();
        switch (operacija) {
            case Util.OPERACIJA_VRATI_SVA_MESTA:
                vratiSvaMesta(soket);
                break;
            case Util.OPERACIJA_SACUVAJ_KORISNIKA:
                Korisnik korisnik = (Korisnik) klijentZahtev.getParametar();
                sacuvajKorisnika(soket, korisnik);
                break;
        }
    }

    private void vratiSvaMesta(Socket soket) throws IOException {
        RadSaBazom rsb = new RadSaBazom();
        List<Mesto> lista = null;
        String poruka = "";
        int status = Util.STATUS_ODGOVOR_SERVER_OK;
        try {
            lista = rsb.vratiSvaMesta();
        } catch (SQLException ex) {
            Logger.getLogger(Server.class.getName()).log(Level.SEVERE, null, ex);
            status = Util.STATUS_ODGOVOR_SERVER_NOT_OK;
            poruka = ex.getMessage();
        }
        //pravimo server odgovor
        TOServerOdgovor serverOdgovor = new TOServerOdgovor();
        serverOdgovor.setRezultat(lista);
        serverOdgovor.setStatusIzvrsenja(status);
        serverOdgovor.setPoruka(poruka);
        //saljemo odgovor klijentu
        posaljiOdgovor(serverOdgovor, soket);
    }

    private void posaljiOdgovor(TOServerOdgovor serverOdgovor, Socket soket) throws IOException {
        ObjectOutputStream outputStream
                = new ObjectOutputStream(soket.getOutputStream());
        outputStream.writeObject(serverOdgovor);
    }

    private void sacuvajKorisnika(Socket soket, Korisnik korisnik) throws IOException {
        RadSaBazom rsb = new RadSaBazom();
        String poruka = "Uspesno sacuvano";
        int status = Util.STATUS_ODGOVOR_SERVER_OK;
        try {
            rsb.sacuvajKorisnika(korisnik);
        } catch (Exception ex) {
            Logger.getLogger(Server.class.getName()).log(Level.SEVERE, null, ex);
            status = Util.STATUS_ODGOVOR_SERVER_NOT_OK;
            poruka = ex.getMessage();
        }
        //pravimo server odgovor
        TOServerOdgovor serverOdgovor = new TOServerOdgovor();
        serverOdgovor.setRezultat(null);
        serverOdgovor.setStatusIzvrsenja(status);
        serverOdgovor.setPoruka(poruka);
        //saljemo odgovor klijentu
        posaljiOdgovor(serverOdgovor, soket);
    }

}
