/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package logika;

import db.DatabaseBroker;
import domen.Artikal;
import domen.Korisnik;
import domen.Mesto;
import java.util.List;
import podaci.KolekcijaKorisnika;
import podaci.KolekcijaMesta;

public class Kontroler {

    private static Kontroler instance;
    private DatabaseBroker dbbr;

    private Kontroler() {
        dbbr = new DatabaseBroker();
    }

    public static Kontroler getInstance() {
        if (instance == null) {
            instance = new Kontroler();
        }
        return instance;
    }

    public void dodajKorisnika(Korisnik korisnik) throws Exception {
        
        try {
            dbbr.uspostaviKonekciju();
        } catch (Exception e) {
            e.printStackTrace();
            throw new Exception("Greska prilikom uspostavljanja konekcije nad bayzom!\n" + e);
        }
        try {
            dbbr.kreirajNovogKorisnikaBoljiNacin(korisnik);
            dbbr.commitTransakcije();
        } catch (Exception e) {
            dbbr.rollbackTransakcije();
            e.printStackTrace();
            throw new Exception("Greska prilikom kreiranja novog korisnika u bazi!\n" + e);
        }
        try {
            dbbr.zatvoriKonekciju();
        } catch (Exception e) {
            e.printStackTrace();
            throw new Exception("Greska prilikom zatvaranje konekcije na bazu!\n" + e);
        }
    }

    public List<Mesto> vratiSvaMesta() throws Exception {
        List<Mesto> mesta = null;
        try {
            dbbr.uspostaviKonekciju();
        } catch (Exception e) {
            e.printStackTrace();
            throw new Exception("Greska prilikom uspostavljanja konekcije nad bayzom!\n" + e);
        }
        
        try {
            mesta = dbbr.vratiSvaMesta();
            dbbr.commitTransakcije();
        } catch (Exception e) {
            dbbr.rollbackTransakcije();
            e.printStackTrace();
            throw new Exception("Greska prilikom ucitavanja mesta iz baze!\n" + e);
        }
        try {
            dbbr.zatvoriKonekciju();
        } catch (Exception e) {
            e.printStackTrace();
            throw new Exception("Greska prilikom zatvaranje konekcije na bazu!\n" + e);
        }
        return mesta;
    }

    public List<Korisnik> vratiSveKorisnike() throws Exception {
        List<Korisnik> korisnici = null;
        try {
            dbbr.uspostaviKonekciju();
        } catch (Exception e) {
            e.printStackTrace();
            throw new Exception("Greska prilikom uspostavljanja konekcije nad bayzom!\n" + e);
        }
        try {
            korisnici = dbbr.vratiSveKorisnike();
            dbbr.commitTransakcije();
        } catch (Exception e) {
            dbbr.rollbackTransakcije();
            e.printStackTrace();
            throw new Exception("Greska prilikom ucitavanja korisnika iz baze!\n" + e);
        }
        try {
            dbbr.zatvoriKonekciju();
        } catch (Exception e) {
            e.printStackTrace();
            throw new Exception("Greska prilikom zatvaranje konekcije na bazu!\n" + e);
        }
        return korisnici;
    }

    public void dodajListuKorisnika(List<Korisnik> korisnici) throws Exception {
        try {
            dbbr.uspostaviKonekciju();
        } catch (Exception e) {
            e.printStackTrace();
            throw new Exception("Greska prilikom uspostavljanja konekcije nad bayzom!\n" + e);
        }
        try {
            for (Korisnik korisnik : korisnici) {
                dbbr.kreirajNovogKorisnikaBoljiNacin(korisnik);
            }
            dbbr.commitTransakcije();
        } catch (Exception e) {
            dbbr.rollbackTransakcije();
            e.printStackTrace();
            throw new Exception("Greska prilikom cuvanja liste novih korisnika u bazi!\n" + e);
        }
        try {
            dbbr.zatvoriKonekciju();
        } catch (Exception e) {
            e.printStackTrace();
            throw new Exception("Greska prilikom zatvaranje konekcije na bazu!\n" + e);
        }
    }
    
    public List<Artikal> vratiSveArtikle() throws Exception {
        List<Artikal> artikli = null;
        try {
            dbbr.uspostaviKonekciju();
        } catch (Exception e) {
            e.printStackTrace();
            throw new Exception("Greska prilikom uspostavljanja konekcije nad bayzom!\n" + e);
        }
        try {
            artikli = dbbr.vratiSveArtikle();
            dbbr.commitTransakcije();
        } catch (Exception e) {
            dbbr.rollbackTransakcije();
            e.printStackTrace();
            throw new Exception("Greska prilikom ucitavanja artikala iz baze!\n" + e);
        }
        try {
            dbbr.zatvoriKonekciju();
        } catch (Exception e) {
            e.printStackTrace();
            throw new Exception("Greska prilikom zatvaranje konekcije na bazu!\n" + e);
        }
        return artikli;
    }
}
