/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package logika;

import db.DatabaseBroker;
import domen.Artikal;
import domen.Korisnik;
import domen.Mesto;
import domen.Racun;
import domen.StavkaRacuna;
import java.io.EOFException;
import java.util.ArrayList;
import java.util.List;
import podaci.KolekcijaKorisnika;
import podaci.KolekcijaMesta;

/**
 *
 * @author student1
 */
public class Kontroler {
    
    private static Kontroler instance;
    private DatabaseBroker dbbr;
    
    private Kontroler () {
        dbbr=new DatabaseBroker();
    }
    
    public static Kontroler getInsance() {
        if (instance == null) {
            instance = new Kontroler();
        }
        return instance;
    }
    
    public List<Mesto> vratiSvaMesta() throws Exception {
        List<Mesto> mesta=null;
        try{
            dbbr.uspostaviKonekciju();
            mesta=dbbr.vratiSvaMesta();
            dbbr.commitTransakcije();
            dbbr.zatvoriKonekciju();
            
        }catch(Exception e){
            dbbr.rollbackTransakcije();
            e.printStackTrace();
            throw new Exception("Greska prilikom citanja mesta iz baze!\n"+e);
        }
        return mesta;
    }
    
    public void dodajKorisnika(Korisnik korisnik) throws Exception {
        try{
            dbbr.uspostaviKonekciju();
            dbbr.kreirajNovogKorisnikaBoljiNacin(korisnik);
            dbbr.zatvoriKonekciju();
        }catch(Exception e){
            e.printStackTrace();
            throw new Exception("Greska prilikom kreiranja novog korisnika u bazi!\n"+e);
        }
    }
    
    public void dodajListuKorisnika(List<Korisnik> korisnici) throws Exception {
        try{
            dbbr.uspostaviKonekciju();
        }catch(Exception e){
            e.printStackTrace();
            throw new Exception("Greska prilikom uspostavljanja konekcije!\n"+e.getMessage());
        }
        try{
            for (Korisnik korisnik : korisnici) {
                dbbr.kreirajNovogKorisnikaBoljiNacin(korisnik);
            }
            dbbr.commitTransakcije();
        }catch(Exception e){
            dbbr.rollbackTransakcije();
            e.printStackTrace();
            throw new Exception("Greska prilikom cuvanja liste novih korisnika u bazi!\n"+e);
        }
        try{
            dbbr.zatvoriKonekciju();
        }catch(Exception e){
            e.printStackTrace();
            throw new Exception("Greska prilikom zatvaranja konekcije na bazu!\n"+e);
        }
    }
    
    public List<Korisnik> vratiSveKorisnike() throws Exception{
        List<Korisnik> korisnici=null;
        try{
            dbbr.uspostaviKonekciju();
        }catch(Exception e){
            e.printStackTrace();
            throw new Exception("Greska prilikom uspostavljanja konekcije!\n"+e.getMessage());
        }
        try{
            korisnici=dbbr.vratiSveKorisnike();
            dbbr.commitTransakcije();
        }catch(Exception e){
            dbbr.rollbackTransakcije();
            e.printStackTrace();
            throw new Exception("Greska prilikom citanja korisnika iz baze!\n"+e);
        }
        try{
            dbbr.zatvoriKonekciju();
        }catch(Exception e){
            e.printStackTrace();
            throw new Exception("Greska prilikom zatvaranja konekcije na bazu!\n"+e);
        }
        return korisnici;
    }
    
    public List<Artikal> vratiSveArtikle() throws Exception {
        List<Artikal> artikli=new ArrayList<>();
        try{
            dbbr.uspostaviKonekciju();
        }catch(Exception e){
            e.printStackTrace();
            throw new Exception("Greska prilikom uspostavljanja konekcije!\n"+e.getMessage());
        }
        try{
            artikli=dbbr.vratiSveArtikle();
            dbbr.commitTransakcije();
        }catch(Exception e){
            dbbr.rollbackTransakcije();
            e.printStackTrace();
            throw new Exception("Greska prilikom citanja liste artikala iz baze!\n"+e);
        }
        try{
            dbbr.zatvoriKonekciju();
        }catch(Exception e){
            e.printStackTrace();
            throw new Exception("Greska prilikom zatvaranja konekcije na bazu!\n"+e);
        }
        return artikli;
    }

    public void sacuvajRacun(Racun racun) throws Exception {
        try{
            dbbr.uspostaviKonekciju();
        }catch(Exception e){
            e.printStackTrace();
            throw new Exception("Greska prilikom uspostavljanja konekcije!\n"+e.getMessage());
        }
        try{
            dbbr.kreirajNoviRacun(racun);
            for (StavkaRacuna stavkaRacuna : racun.getStavke()) {
                dbbr.kreirajNovuStavkuRacuna(stavkaRacuna);
            }
            dbbr.commitTransakcije();
        }catch(Exception e){
            dbbr.rollbackTransakcije();
            e.printStackTrace();
            throw new Exception("Greska prilikom cuvanja racuna u bazi!\n"+e);
        }
        try{
            dbbr.zatvoriKonekciju();
        }catch(Exception e){
            e.printStackTrace();
            throw new Exception("Greska prilikom zatvaranja konekcije na bazu!\n"+e);
        }
    }
}
